/*	$NetBSD: vrtc.c,v 1.2 2019/11/10 21:16:33 chs Exp $	*/
/*	$OpenBSD: vofdisk.c,v 1.1 2008/03/08 19:19:43 kettenis Exp $	*/
/*
 * Copyright (c) 2008 Mark Kettenis
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <sys/param.h>
#include <sys/buf.h>
#include <sys/device.h>
#include <sys/conf.h>
#include <sys/disklabel.h>
#include <sys/disk.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/systm.h>
#include <sys/proc.h>

#include <machine/autoconf.h>
#include <machine/hypervisor.h>
#include <machine/openfirm.h>

#include <sparc64/dev/vbusvar.h>


struct vofdisk_softc {
        device_t sc_dev;
	/* int sc_phandle; */
	/* int sc_unit; */
	/* int sc_flags; */
	struct disk sc_dk;
	/* int sc_ihandle; */
	/* u_long max_transfer; */
};

dev_type_open(vofdisk_open);
dev_type_close(vofdisk_close);
dev_type_read(vofdisk_read);
dev_type_write(vofdisk_write);
dev_type_ioctl(vofdisk_ioctl);
dev_type_strategy(vofdisk_strategy);
dev_type_dump(vofdisk_dump);
dev_type_size(vofdisk_size);

static void vofminphys(struct buf *);

struct dkdriver vofdisk_dkdriver = {
	.d_strategy = vofdisk_strategy,
	.d_minphys = vofminphys
};

int	vofdisk_match(device_t, cfdata_t, void *);
void	vofdisk_attach(device_t, device_t, void *);

CFATTACH_DECL_NEW(vofdisk, sizeof(device_t),
    vofdisk_match, vofdisk_attach, NULL, NULL);

int
vofdisk_match(device_t parent, cfdata_t match, void *aux)
{
	struct vbus_attach_args *va = aux;

	if (strcmp(va->va_name, "disk") == 0)
		return (1);

	
	return (0);
}

void
vofdisk_attach(device_t parent, device_t self, void *aux)
{

	struct vofdisk_softc *vof = device_private(self);

	disk_init(&vof->sc_dk, device_xname(vof->sc_dev), &vofdisk_dkdriver);
	disk_attach(&vof->sc_dk);

	dkwedge_discover(&vof->sc_dk);
	
	printf("\n");
}

void
vofdisk_strategy(struct buf *bp)
{
  panic("vofminphys()");
#if 0  
	struct ofdisk_softc *of =
		device_lookup_private(&ofdisk_cd, DISKUNIT(bp->b_dev));
	struct partition *p;
	u_quad_t off;
	int read;
	int (*OF_io)(int, void *, int);
	daddr_t blkno = bp->b_blkno;

	bp->b_resid = 0;
	if (bp->b_bcount == 0)
		goto done;

	OF_io = bp->b_flags & B_READ ? OF_read : 
		(int(*)(int, void*, int))OF_write;

	if (DISKPART(bp->b_dev) != RAW_PART) {
		if (bounds_check_with_label(&of->sc_dk, bp, 0) <= 0) {
			bp->b_resid = bp->b_bcount;
			goto done;
		}
		p = &of->sc_dk.dk_label->d_partitions[DISKPART(bp->b_dev)];
		blkno = bp->b_blkno + p->p_offset;
	}

	disk_busy(&of->sc_dk);

	off = (u_quad_t)blkno * DEV_BSIZE;
	read = -1;
	do {
		if (OF_seek(of->sc_ihandle, off) < 0)
			break;
		read = OF_io(of->sc_ihandle, bp->b_data, bp->b_bcount);
	} while (read == -2);

	if (read < 0) {
		bp->b_error = EIO;
		bp->b_resid = bp->b_bcount;
	} else
		bp->b_resid = bp->b_bcount - read;

	disk_unbusy(&of->sc_dk, bp->b_bcount - bp->b_resid,
	    (bp->b_flags & B_READ));

done:
	biodone(bp);
#endif	
}


static void
vofminphys(struct buf *bp)
{
  panic("vofminphys()");
#if 0  
	struct ofdisk_softc *of =
		device_lookup_private(&ofdisk_cd, DISKUNIT(bp->b_dev));

	if (bp->b_bcount > of->max_transfer)
		bp->b_bcount = of->max_transfer;
#endif	
}

